; COUNT.ASM 
;
; Counts 0-9 on a 7-segment display using the watchdog as a timing
; source.
;
; A common anode 7-segment LED display is attached as follows:
;
;    RA0 segment a
;    RA1 segment b
;    RA2 segment c
;    RA3 segment d
;    RA4 segment e
;    RB4 segment f
;    RB5 segment g
;
; A port pin must be pulled low to turn on a segment.   
        
        LIST            P=16C84 ; works for 16F84 without change 
        ERRORLEVEL      -302    ; suppress bank selection messages
        __CONFIG        3FFDH   ; XT oscillator and watchdog enabled

PCL     EQU     2
STATUS  EQU     3  
PORTA   EQU     5
PORTB   EQU     6 
TRISA   EQU     85H
TRISB   EQU     86H
OPTREG  EQU     81H

C       EQU     0
Z       EQU     2
RP0     EQU     5

COUNT   EQU     0CH             ; current count
SEGS    EQU     0DH             ; one bit per segment: "-gfedcba"

        ORG     0

        CLRF    COUNT           ; initialisation
        CLRF    PORTA
        CLRF    PORTB           ; all LEDS off
        BSF     STATUS,RP0      ; select register bank 1  
        CLRF    TRISA
        CLRF    TRISB           ; both ports are all outputs
        MOVLW   0EH
        MOVWF   OPTREG          ; assign prescaler (1:64) to WDT
        BCF     STATUS,RP0      ; reselect bank 0

START   MOVFW   COUNT           ; get COUNT
        CALL    DISPLAY         ; and display
        SLEEP                   ; wait for watchdog timeout
        INCF    COUNT,F         ; bump COUNT
        MOVF    COUNT,W
        SUBLW   9               ; compare COUNT with 9
        BTFSC   STATUS,C
         GOTO    START          ; if less than or equal goto START
        CLRF    COUNT           ; else COUNT=0
        GOTO    START           ; and goto START

DISPLAY CALL    SEGTAB          ; convert W to segment form
        MOVWF   SEGS            ; and store
        ANDLW   1FH             ; extract PORTA segments
        MOVWF   PORTA           ; and display
        RRF     SEGS,W          
        ANDLW   30H             ; extract PORTB segments
        MOVWF   PORTB           ; and display
        RETURN

SEGTAB  ANDLW   0FH             ; convert a number in W to segment form
        ADDWF   PCL,F
        DT      40,79,24,30,19,12,02,78,00,10
        DT      7F,7F,7F,7F,7F,7F

        END

