; COUNT1.ASM 
;
; Cycles through 0 to 9 on a 7-segment display; a switch is used
; to increment the count.
;
; A common-anode 7-segment LED display is attached via 470 ohm resistors 
; as follows:
;
;    RA0 segment a
;    RA1 segment b
;    RA2 segment c
;    RA3 segment d
;    RA4 segment e
;    RB4 segment f
;    RB5 segment g
;
; A port pin must be pulled low to turn on a segment.   
;
; RB7 can be grounded via a push-to-make switch.
        
        LIST            P=16C84 ; works for 16F84 without change
        ERRORLEVEL      -302    ; suppress bank selection messages
        __CONFIG        3FF9H   ; XT oscillator

PCL     EQU     2
STATUS  EQU     3   
PORTA   EQU     5
PORTB   EQU     6 
TRISA   EQU     85H
TRISB   EQU     86H
OPTREG  EQU     81H

C       EQU     0
Z       EQU     2
RP0     EQU     5

COUNT   EQU     0CH
SEGS    EQU     0DH
DELCNT  EQU     0EH
        
        ORG     0

INIT    CLRF    COUNT           ; COUNT = 0
        CLRF    PORTA
        CLRF    PORTB           ; All LEDs off
        BSF     STATUS,RP0      ; select register bank 1
        CLRF    TRISA
        MOVLW   80H
        MOVWF   TRISB           ; all outputs except RB7
        CLRF    OPTREG          ; enable PORTB pullups
        BCF     STATUS,RP0      ; select register bank 1
        MOVF    COUNT,W         ; get COUNT
        CALL    DISPLAY         ; and display

MAIN    BTFSC   PORTB,7         ; switch closed?
         GOTO    MAIN           ; no
        INCF    COUNT,F         ; yes, bump COUNT
        MOVF    COUNT,W
        SUBLW   9               ; compare COUNT with 9
        BTFSS   STATUS,C
         CLRF    COUNT          ; if greater COUNT=0
        MOVF    COUNT,W
        CALL    DISPLAY
        CALL    DELAY           ; wait for switch to settle
CHKSWO  BTFSS   PORTB,7         ; switch open?
         GOTO    CHKSWO         ; no 
        CALL    DELAY
        BTFSS   PORTB,7         ; still open?
         GOTO    CHKSWO         ; no
        GOTO    MAIN

DISPLAY CALL    SEGTAB          ; convert W to segment form
        MOVWF   SEGS            ; and store 
        ANDLW   1FH             ; retain only PORTA part
        MOVWF   PORTA           ; and display
        RRF     SEGS,W
        ANDLW   30H             ; retain only PORTB part
        MOVWF   PORTB           ; and display
        RETURN

SEGTAB  ANDLW   0FH             ; convert W to segment form
        ADDWF   PCL,F
        DT      40,79,24,30,19,12,02,78,00,10
        DT      7F,7F,7F,7F,7F,7F
        
DELAY   MOVLW   D'20'           ; roughly 20ms delay @ 4MHz
        MOVWF   DELCNT
        CLRW
DELAY1  ADDLW   1
        BTFSS   STATUS,Z
         GOTO    DELAY1
        DECFSZ  DELCNT,F
         GOTO    DELAY1
        RETURN

        END
